import os

import wikifile
from wikifile.wikiFile import WikiFile
from wikifile.metamodel import Context, Topic, UML, Property
from wikifile.smw import SMWPart, SMW, Form, ListOf, Query
import json
import sys
import jinja2
from distutils.sysconfig import get_python_lib
from wikifile.toolbox import Toolbox


class WikiRender(Toolbox):
    """
    Provides functions to render json data to wiki files
    """

    def __init__(self, template_env=None, debug=False):
        ''' constructor
        '''
        self.debug=debug  # debug state before args.debug is available
        super(WikiRender, self).__init__()
        if template_env is None:
            template_env = WikiRender.getTemplateEnv()
        self.template_env = template_env

    def main(self, argv=None):
        if argv is None:
            argv = sys.argv
        
        # Modes of operation
        UPDATE_TEMPLATES_MODE = "update_templates"
        CREATE_FILE_MODE = "create"
        GENERATE_ENTITY_PAGES = "generate_entity_pages"
        modes = [UPDATE_TEMPLATES_MODE, CREATE_FILE_MODE, GENERATE_ENTITY_PAGES]
       
        # Setup argument parser
        self.getParser()
        try:
            # Process arguments
            args = self.parser.parse_args(argv)
            self.debug=args.debug
            if args.mode not in modes:
                raise Exception(f"Please select of of the operation modes: {modes}")
            if args.mode == GENERATE_ENTITY_PAGES:
                # Check if necessary parameters are set
                if not args.properties_file or not args.topic_file:
                    raise Exception("The parameters --properties and --topics must be defined for this mode")
                if args.templates_folder:
                    # update templateEnv
                    self.template_env = self.getTemplateEnv(additional_loader=args.template_folder)
                properties_json = ""
                print(1)
                with open(args.properties_file) as json_file:
                    properties_json = json_file.read()
                topic_json = ""
                with open(args.topic_file) as json_file:
                    topic_json = json_file.read()
                topic = Topic.from_wiki_json(topic_json=topic_json, prop_json=properties_json)
                # Generate and save the entity pages
                self.generateTopic(topic, args.backupPath, args.overwrite)
            elif args.mode == UPDATE_TEMPLATES_MODE:
                data = {}
                if "data_input" in args:
                    data = json.loads(args.data_input)
                elif args.stdin:
                    data = json.load(sys.stdin)
                else:
                    pass
                template_data = data.get(args.template)
                self.update_or_create_templates(template_data, name_id="pageTitle" , template_name=args.template, backup_path=args.backupPath ,overwrite=args.overwrite)


            if args.mode == CREATE_FILE_MODE:
                # ToDo
                pass


        except KeyboardInterrupt:
            ### handle keyboard interrupt ###
            return 1
        except Exception as e:
            raise e

    def getParser(self):
        # Setup argument parser
        super().getParser() # setup default parser arguments
        if self.parser is None:
            raise AttributeError("parser of this object should be defined at this point.")
        self.parser.add_argument("-m", "--mode", dest="mode",
                                 help="Select a mode.\n\tupdate_templates: updates the wikifiles at the provided location with the provided data\n\tcreate: creates a wikifile with the given data.",
                                 required=True)
        self.parser.add_argument('--properties', dest="properties_file", help="Json file containing properties")
        self.parser.add_argument('--topic', dest="topic_file", help="Json file containing topics")
        self.parser.add_argument('-f', dest="overwrite", action='store_true', default=False,
                                 help='If true template arguments will be overwritten with the given data if present')
        self.parser.add_argument("-t", "--template", dest="template",
                                 help="Select a template in which the data is being rendered")
        self.parser.add_argument("--templates", dest="templates_folder",
                                 help="Path to additional templates")
        self.parser.add_argument("--data", dest="data_input",
                                 help="Json file that should be used as data input")

    @staticmethod
    def getTemplateEnv(template_dir: str='/templates', additional_loader=None):
        loader = []
        if additional_loader is not None:
            loader.append(jinja2.FileSystemLoader(searchpath=additional_loader))
        script_dir = os.path.dirname(wikifile.__file__) + "/.."
        loader.append(jinja2.FileSystemLoader(script_dir + template_dir))
        loader.append(jinja2.ModuleLoader(get_python_lib() + template_dir))
        templateLoader = jinja2.ChoiceLoader(loader)
        templateEnv = jinja2.Environment(loader=templateLoader)
        return WikiRender.extend_template_env(templateEnv)

    @staticmethod
    def extend_template_env(template_env):
        """Add SMW classes and MetaModel classse to the template environment along with other use full functions"""
        template_env.globals['SMW'] = SMW
        template_env.globals['SMWPart'] = SMWPart
        template_env.globals['Form'] = Form
        template_env.globals['ListOf'] = ListOf
        template_env.globals['Query'] = Query
        template_env.globals['UML'] = UML
        template_env.globals['Property'] = Property
        template_env.globals['Topic'] = Topic
        template_env.globals['Context'] = Context
        # Further functionc
        template_env.globals['map'] = map
        return template_env

    def render_template(self, template_name: str, data: dict, exclude_keys: list = None):
        """
        Render the given data to a template
        Example:
            template_name = "Event"
            data = {"name": "SMWCon", "date": "2020", "Filename": "SMWCon 2020.wiki"}
            exclude_keys = ["Filename"]

            Results in:
            {{Event
            |name=SMWCon
            |date=2020
            }}
        Args:
            template_name: Name of the template
            data: Dictionary the keys are used translated as template arguments and the values as argument value
            exclude_keys: list of keys that should not be included in the template

        Returns:

        """
        if exclude_keys is not None:
            data = {x: data[x] for x in data if x not in exclude_keys}
        try:
            template_template = self.template_env.get_template("template.jinja")
            return template_template.render(name=template_name, properties=data)
        except Exception as e:
            print(e)
        return None

    def generateTopic(self, topic: Topic, path: str, overwrite: bool):
        """
        Generate all technical pages of the given topic and save them as wiki page at the given path
        Args:
            topic: topic for which the pages should be generated
            path: path to the location where the pages should be stored
            overwrite: If true the generated pages will overwrite existing pages. Otherwise only missing pages will be stored
        """
        if self.debug:
            print("generating topic %s" % topic.name)
        for part, smwPart in SMWPart.getAll(self).items():
            if self.debug:
                print(f"generating {smwPart.get_page_name(topic)}")
            page = smwPart.render_page(topic)
            wiki_file = WikiFile(smwPart.get_page_name(topic), path, wiki_render=self, wikiText=page, debug=self.debug)
            wiki_file.save_to_file(overwrite)

    def update_or_create_templates(self,
                                   data: list,
                                   name_id: str,
                                   template_name: str,
                                   backup_path: str,
                                   exclude_keys: list = None,
                                   overwrite=False):
        """
        Iterates over the given data and updates or creates the corresponding wikifile. The parameter wiki_id is used
        as the filename.
        Args:
            data: json data containing the information for the templates
            name_id: Name of the key that contains the page name
            template_name: Name of the template
            template_env:
            backup_path: path to the directory were the wiki files are stored
            exclude_keys: List of keys that is included in the data but should be excluded
            overwrite: If True arguments that are already defined in the template are overwritten by the given data. If False only missing data is added
        Returns:

        """
        for page in data:
            # check if page exists
            if name_id not in page:
                continue
            page_name = page[name_id]
            # Remove excluded keys
            for exclude in exclude_keys:
                del page[exclude]
            wiki_file = WikiFile(page_name, backup_path, wiki_render=self)
            if wiki_file.wikiText is None:
                # create page and store it
                page_content = self.render_template(template_name, page)
                WikiFile(path=backup_path, name=page_name, wiki_render=self, wikiText=page_content).save_to_file(
                    overwrite)
            else:
                # update existing template or create new one
                wiki_file.add_template(template_name, page)
                wiki_file.save_to_file(overwrite=overwrite)

def main_module_call():
    wikirender = WikiRender()
    wikirender.main(sys.argv[1:])
    sys.exit()

if __name__ == '__main__':
    wikirender=WikiRender()
    wikirender.main(sys.argv[1:])
    sys.exit()




from wikifile.wikiFile import WikiFile
import json
import logging
import os
import sys

from wikifile.toolbox import Toolbox


class WikiExtract(Toolbox):
    """
    Provides methods to extract data from wiki files.
    """
    def __init__(self, debug=False):
        super(WikiExtract, self).__init__()

    def main(self, argv=None):
        if argv is None:
            argv = sys.argv

        self.getParser()
        try:
            # Process arguments
            args = self.parser.parse_args(argv)
            if args.debug:
                logging.basicConfig(level=logging.DEBUG, stream=sys.stdout)
            else:
                logging.basicConfig(stream=sys.stdout, level=logging.INFO)

            file_parameters = [args.stdin, args.pages, args.file_list]
            if len(file_parameters) - (file_parameters.count(None) + file_parameters.count(False)) > 1:
                logging.error(
                    "Multiple file selection options were used. Please use only one or none to select all files in the backup folder.")
                raise Exception("Invalid parameters")

            res_templates = WikiExtract.extract_templates(args.template,
                                                          stdIn=args.stdin,
                                                          file_list=args.file_list,
                                                          page_titles=args.pages,
                                                          backup_path=args.backupPath,
                                                          add_file_name=args.file_name_id)
            print(res_templates)


        except KeyboardInterrupt:
            ### handle keyboard interrupt ###
            return 1
        except Exception as e:
            print(e)

    def getParser(self):
        # Setup argument parser
        super().getParser() # setup default parser arguments
        if self.parser is None:
            raise AttributeError("parser of this object should be defined at this point.")
        self.parser.add_argument("-m", "--mode", dest="mode",
                                 help="Select a mode.\n\tupdate_templates: updates the wikifiles at the provided location with the provided data\n\tcreate: creates a wikifile with the given data.",
                                 required=True)
        # Add parser arguments
        self.parser.add_argument("-t", "--template", dest="template",
                                 help="Select a template in which the data is being rendered", required=True)
        self.parser.add_argument("-id", "--file_name_id", dest="file_name_id",
                                 help="Name of the key in which the file name is stored.")
        self.parser.add_argument("--listFile", dest="file_list",
                                 help="List of pages form which the data should be extracted", required=False)

    @staticmethod
    def extract_templates(template_name: str, stdIn, page_titles, file_list, backup_path, add_file_name):
        """
        Extracts template data of the template template_name from the given files/location and returns the data as json.
        Args:
            template_name: name of the template that should be extracted
            stdIn:
            page_titles:
            file_list:
            backup_path:
            add_file_name: If defined this value will be used as key to store the filename. Should be used wisely to not interfere with regular template arguments.

        Returns:

        """
        if stdIn:
            l = []
            backup_path = os.path.dirname(page_titles[0].strip())
            pageTitlesfix = []
            for i in page_titles:
                pageTitlesfix.append(os.path.basename(i.strip().replace('.wiki', '')))
            page_titles = pageTitlesfix
        elif file_list is not None:
            f = open(file_list, 'r')
            allx = f.readlines()
            page_titles = []
            for i in allx:
                page_titles.append(os.path.basename(i.strip()).replace('.wiki', ''))
        else:
            if backup_path is None:
                logging.warning("No backup path is defined. Please provide a path to the location were the wiki-files are stored.")
            imageBackupPath = "%s/images" % backup_path
            if page_titles is None:
                page_titles = []
                for path, subdirs, files in os.walk(backup_path):
                    for name in files:
                        filename = os.path.join(path, name)[len(backup_path) + 1:]
                        if filename.endswith(".wiki"):
                            page_titles.append(filename[:-len(".wiki")])
        total = len(page_titles)
        logging.debug(f"extracting templates from {total} wikifiles.")
        res = []
        for file in page_titles:
            wikiFile = WikiFile(file, backup_path)
            template = wikiFile.extract_template(template_name)
            if template is not None:
                if add_file_name is not None:
                    template[add_file_name] = file
                res.append(template)
        return json.dumps({"data": res}, default=str, indent=3)

def main_module_call():
    wikiextract = WikiExtract(is_module_call=True)
    wikiextract.main(sys.argv[1:])
    sys.exit()

if __name__ == '__main__':
    wikiextract = WikiExtract(is_module_call=True)
    wikiextract.main(sys.argv[1:])
    sys.exit()



import os
import wikitextparser as wtp
from wikitextparser import Template


class WikiFile:
    '''
    Provides methods to modify, query, update and save wiki files.
    '''

    def __init__(self, name, path, wiki_render, wikiText: str = None, debug=False):
        """

        Args:
            name: Name of the file
            path: Path to file location
            wiki_render:
            wikiText: WikiPage contend as string. If defined the file content will loaded and this value will be used instead
        """
        self.file_name = name
        self.file_path = path
        self.debug = debug
        if wikiText is None:
            self.wikiText = self.get_wikiText(self.file_name, self.file_path)
        else:
            self.wikiText = wtp.parse(wikiText)
        self.wiki_render = wiki_render

    def save_to_file(self, overwrite=False):
        """
        Save the given data in a file.
        Args:
            overwrite: If True existing files will be over written and if the path does not exist the missing folder will be created. Otherwise, only missing files will be saved.
        Returns:

        """
        wiki_file_path = WikiFile.get_wiki_path(self.file_path, self.file_name)
        mode = "w"
        if overwrite:
            mode = 'w'
            os.makedirs(os.path.dirname(wiki_file_path), exist_ok=True)
        else:
            if os.path.isfile(wiki_file_path):
                # file already exists
                if self.debug:
                    print(f"File {wiki_file_path} exists \t-> Generated page not saved. To save also existing pages use -f to overwrite them.", )
                    return
        with open(wiki_file_path, mode=mode) as f:
            f.write(str(self.wikiText))
        if self.debug:
            print(f"{self.file_name} saved to {wiki_file_path}")

    def get_wikiText(self, name, path):
        """
        find the wiki file by name and return it as parsed object.
        Args:
            name: Name of the file
            path: Path to the file

        Returns:
            WikiText object
        """
        fname = WikiFile.get_wiki_path(path, name)
        if os.path.isfile(fname):
            with open(fname, mode='r') as file:
                page = file.read()
                parsed_page = wtp.parse(page)
                return parsed_page
        return None

    def get_template(self, template_name: str):
        if self.wikiText is None or self.wikiText.templates is None:
            # Wikifile has no templates
            return None
        for template in self.wikiText.templates:
            name = WikiFile.get_template_name(template.name)
            if name == WikiFile.get_template_name(template_name):
                return template
        return None

    @staticmethod
    def get_template_name(template_name: str):
        name = template_name.replace('\n', '')
        name = name.lstrip()
        name = name.rstrip()
        return name

    def update_template(self, template_name: str, args, force=False):
        template = self.get_template(template_name)
        for key, value in args.items():
            if template.has_arg(key):
                # update argument
                if force:
                    template.del_arg(key)
                    template.set_arg(key, value)
                else:
                    pass
            else:
                template.set_arg(key, value)

    @staticmethod
    def update_arguments(template: Template, args, overwrite=False):
        for key, value in args.items():
            if template.has_arg(key):
                # update argument
                if overwrite:
                    template.del_arg(key)
                    template.set_arg(key, value)
                else:
                    pass
            else:
                if value is not None:
                    template.set_arg(key, value)

    def add_template(self, template_name: str, data: dict, overwrite=False):
        template = self.get_template(template_name)
        if template is None:
            # create template
            self.wikiText.insert(0, self.wiki_render.render_template(template_name, data))
        else:
            WikiFile.update_arguments(template, data, overwrite)

    def extract_template(self, name: str):
        """
        Extracts the template data and returns it as dict
        Args:
            name: name of the template that should be extracted

        Returns:
            Returns template content as dict
        """
        template = self.get_template(name)
        if template is None:
            return None
        res = {}
        for arg in template.arguments:
            value = arg.value if not arg.value.endswith("\n") else arg.value[:-1]
            res[arg.name] = value
        return res

    @staticmethod
    def get_wiki_path(path: str, name: str):
        return f"{path}/{name}.wiki"
